RtsDefine(['rtsmvc', 'model/Media'], function() {

	(function(ns) {

		// RtsDefine model for playlist types
		ns.PlaylistType = ns.Model.extend({
			defAttr : {
				id : {
					validation : [ 'RtsRequired' ]
				},
				shortName : {
					validation : [ 'RtsRequired' ]
				},
				title : {
					validation : [ 'RtsRequired' ]
				},
				areElementsOrderable : {
					validation : [ 'RtsRequired' ]
				},
				areElementsDeletable : {
					validation : [ 'RtsRequired' ]
				},
				areElementsAddable : {
					validation : [ 'RtsRequired' ]
				},
				isDeletable : {
					validation : [ 'RtsRequired' ]
				},
				isEditable: {},
				isArticleDeletable : {
					validation : [ 'RtsRequired' ]
				}
			},
			idAttribute : "id"
		});

		// RtsDefine collection containing playlist types
		ns.CollectionPlaylistTypes = ns.Collection.extend({
			url : "/connecte/typeplaylist/",
			model : ns.PlaylistType,
			
			getItemBy : function(attr, value, onFind){
				
				if(_.isFunction(onFind)){
					if(_.isEmpty(this.items)){

						var c = this._callbacks || (this._pltCallbacks = []);
						c.push({a:attr, v:value, c:onFind});

					} else {

						for(i in this.items){
							if(this.items[i].get(attr) == value){
								
								onFind(this.items[i]);
							}
						}
					}
				}
				return this.items[attr] || null;
			},
			execCallback : function(){
				
				var c = this._pltCallbacks || (this._pltCallbacks = []);

			
				for(var i = 0; i < c.length; i ++){

					
					this.getItemBy(c[i].a, c[i].v, c[i].c);
				}

				delete this._pltCallbacks;

			},
			getItemByShortName : function(value, onFind){
				return this.getItemBy("shortName", value, onFind);
			},
			getItemById : function(value, onFind){
				return this.getItemBy("id", value, onFind);
			},
			awake : function(){
				var b = this;

				this.load(0, 99, {success:function(){
					var cb = b.___onLoad || function() {};


					cb(this);

					b.execCallback();
				}});
			},
			onLoad : function(onLoad){
				this.___onLoad = onLoad;
			}
		});
		ns.PlaylistTypeCollectionShared = ns.Util.makeShareable(ns.CollectionPlaylistTypes, function(c){
			c.awake();
		});
		
		// RtsDefine a playlist model
		ns.Playlist = ns.Model.extend({
			url : "/connecte/playlist/:id",
			idAttribute : "id",
			defAttr : {
				id : {
					validation : [ 'RtsRequired' ]
				},
				title : {
					validation : [ 'RtsRequired' ]
				},
				idPlaylistType : {
					validation : [ 'RtsRequired' ]
				},
				count : {
					validation : [ 'RtsRequired' ]
				},
				article : {
					'default' : function(attrName) {
						return new ns.MediaCollection(this, attrName);
					}
				}
			}
		});
		
		// RtsDefine a playlist collection
		ns.PlaylistCollection = ns.Collection.extend({
			model : ns.Playlist,
			url : "/connecte/playlist/"
		});

	})(rtsmvc);
});
