RtsDefine(['jquery', 'rtsmvc', 'model/Member', 'model/Playlist', 'view/views-menu'], function($) {

	(function(ns){

		// Define environement.
		var env = ns.Util.env;

		
		
		var ControllerDropdownMenu = ns.Controller.extend({
			preLogoutCallbackHooks : [],
			events : {
				"collection:load"             : "onCollectionLoad",
				"click ul.user .profil a"     : "onProfil",
				"click ul.user .disconnect a" : "onDisconnect"
			},
			construct : {
				collection : ns.PlaylistCollection,
				view : ns.ViewDropdownMenu
			},
			init : function(){
				this.view.setEl($('body'));
			},
			open : function(toggleEl) {
				clearTimeout(toggleEl.data("dropdown.timeout"));
				if (toggleEl.hasClass("open")) {
					return;
				}

                var adminLink = ""
                ns.SessionMember().getMember(function(logged, member){
                  if (member.isAdmin()) adminLink = '<li><a href="https://' + env.domains.marts + '/connecte/commentaire/administration">Admin</a></li>';
                }, this);
                
                this.view.setEl(toggleEl.parent());
				this.view.append({ adminLink: adminLink });

				toggleEl.addClass("open");

				var that = this;
				this.getView().domSource.hover(function() {
					clearTimeout(toggleEl.data("dropdown.timeout"));
				}, function() { that.close(toggleEl) });

				this.setCollection(new ns.PlaylistCollection());
				// We want to load only the first three playlists.
				this.collection.load(0, 3);
			},
			close : function(toggleEl) {
				var that = this;

				var timeout = setTimeout(function() {
					that.view.remove();
					toggleEl.removeClass("open");
				}, 500);

				clearTimeout(toggleEl.data("dropdown.timeout"));
				toggleEl.data("dropdown.timeout", timeout);
			},
			onCollectionLoad : function(){
				var el = this.view.find('.playlist');
				el.html('');

				this.collection.each(function(index, item){
					var dropdownMenuItem = new ControllerDropdownMenuItem();

					dropdownMenuItem.getView().setEl(el);
					dropdownMenuItem.setModel(item);
				}, this);

				el.append('<li class="spacer"></li>');
			el.append('<li class="all"><a href="http://' + env.domains.rts + '/profil/"><span class="play2"></span><span>Toutes mes playlists</span></a></li>');
			},
			
			onProfil : function(el, e) {
				ns.Util.openPopup(el.href, (function(x) {return _.bind(x.refresh, x)})(ns.ControllerMenuBarShared()));
				e.preventDefault();
			},
			
			registerPreLogoutCallbackHook : function (fn){
				this.preLogoutCallbackHooks.push(fn);
			},
			
			onDisconnect : function(el, e) {
				// execute preLogoutCallbackHooks
				for (var i = 0, l = this.preLogoutCallbackHooks.length; i < l; i++){
					this.preLogoutCallbackHooks[i]();
				}
				$.ajax({
					dataType: 'json',
					url: 'https://' + env.domains.marts + '/dynamique/deconnexion',
					success: function() {
						// ns.ControllerMenuBarShared().refresh();
						window.location.reload();
					}
				});
				
				e.preventDefault();
			}
		});

		var ControllerDropdownMenuItem = ns.Controller.extend({
			events : {
				"model:set" : "onModelSet"
			},
			construct : {
				model : ns.Playlist,
				view : ns.ViewDropdownMenuItem
			},
			onModelSet : function(){
				this.view.append({
					id : this.model.get('id'),
					title : this.model.get('title'),
					count : this.model.get('article').getSize()
				});
			}
		});

		ns.ControllerMenuBar = ns.Controller.extend({
			events : {
				"model:reload"              : "onModelChange",
				"member:model:reload"       : "onModelChange",
				"model:set"                 : "onModelChange",
				"click .create-account a"   : "onCreateAccount",
				"click a.complete-btn"  	: "onCompleteAccount",
				"click a.login-btn"         : "onLogin"
			},
			construct : {
				view : ns.ViewMenuBar
			},
			initDone : false,
			init : function() {
				this.controllerDropdownMenu = new ControllerDropdownMenu();
				this.viewMenuBar = new ns.ViewMenuBar();
				this.viewMenuLoggedIn = new ns.ViewMenuLoggedIn();
				this.viewMenuInactiveMember = new ns.ViewMenuInactiveMember();
				
				this.setModel(ns.SessionMember());

				ns.SessionMember().getMember(function(logged, member){
					if(logged) this.setModel(member, "member");
				}, this);
				initDone = true;
			},

			inited : function(){
				return initDone;
			},

			// call it like this : 
			// rtsmvc.ControllerMenuBarShared().registerPreLogoutCallbackHook(function(){alert('about to logout');})
			registerPreLogoutCallbackHook : function(fn){
				this.controllerDropdownMenu.registerPreLogoutCallbackHook(fn);
			},

			setCommentCookie : function(activateNewCommand) {
				ns.Util.createCookie("martscomments", activateNewCommand ? 'new' : 'old', 100);
			},

			readCommentCookie : function() {
				ns.Util.getCookie("martscomments");
			},

			isNewCommentActive : function() {
				readCommentCookie() == 'new';
			},

			onModelChange : function() {
				
				this.getModel().getMember(function(logged, member){

					var el = this.view.el;
					try {
						this.view.remove();
					} catch (e) {}

	        		if (member.isLogged()) {
						$(el).addClass("logged");
						$("html").addClass("marts-logged");

						this.setView(this.viewMenuLoggedIn);
						this.view.setEl(el);

						this.view.render({
							nickname: member.getDisplayedName(),
							thumb: member.getThumbUri(ns.Member.THUMB_SMALL) || 
									'//'  + env.domains.marts + '/assets/images/layout/profile-picture/thumb-50x50.jpg'
						});

						var that = this;
						el
							.on("mouseenter", "a.profile-picture", function() {
								that.controllerDropdownMenu.open(el);
							})
							.on("mouseleave", "a.profile-picture", function() {
								that.controllerDropdownMenu.close(el);
							})
							.on("click", "a.profile-picture", function(evt) {
								evt.preventDefault();
							})
							;
						
						ns.Util.createCookie('martsConnectionStatus', 1, 100);

						// setup cookie used for comments integration
						/*if (!this.readCommentCookie() && member.isAdmin()) {
							ns.Util.trace("enable marts comments");
							this.setCommentCookie(true);
            			}*/
					} else {

						ns.Util.createCookie('martsConnectionStatus', 0, 100);


						$("html").removeClass("marts-logged");
						el.removeClass("logged");
						
						el
							.off("mouseenter", "a.profile-picture")
							.off("mouseleave", "a.profile-picture")
							.off("click", "a.profile-picture")
							;

						this.setView(member.isLogged(true) ? this.viewMenuInactiveMember : this.viewMenuBar);
						this.view.setEl(el);
						this.view.render();
					}

					/*if (ns.Util.getURLParameter("martscomments") == "0"){
						ns.Util.trace("disable marts comments");
						this.setCommentCookie(false);
					}*/
					
				}, this);
			},
			onCreateAccount : function(el, e) {
				ns.Util.openPopup(el.href, _.bind(this.refresh, this));
				e.preventDefault();
			},
			onCompleteAccount : function(el, e) {
				ns.Util.openPopup(el.href, _.bind(this.refresh, this));
				e.preventDefault();
			},
			refresh : function() {
				this.getModel().reload();
			},
			onLogin : function(el, e) {
				ns.Util.openPopup(el.href + (env.devname ? "?dev=" + env.devname : ""));
				e.preventDefault();
			}
		});

		ns.ControllerMenuBarShared = ns.Util.makeShareable(ns.ControllerMenuBar);

	})(rtsmvc);

});
