/**
 * Jquery EasyXDM wrapper.
 */
RtsDefine(["jquery-core", "easyXDM"], function($, easyXDM) {
    // Test if easyXDM library is loaded.
    if (!easyXDM)
        throw new Error("Jquery XDM Wrapper needs easyXDM library to be loaded!!");

    // Save original ajax functionality.
    var ajx = $.ajax;

    // Easy XDM rpc.
    var _registry = {};
    function getRpc(domain) {
        if (!_registry[domain]) {
            _registry[domain] = new easyXDM.Rpc(
                {
                    remote: domain + "/cors/",
                    swf: domain + "/assets/javascripts/easyxdm/easyxdm.swf"

                },
                {remote: {request: {}, jqAjax: {}, jqAjaxAbort: {}}}
            );
        }
        
        return _registry[domain];
    }
    
    // Constants.
    var rheaders = /^(.*?):[ \t]*([^\r\n]*)\r?$/mg;
    var uid = 0;
    
    // Extend jQuery.
    
    var jQueryAjaXDM = function(url, options) {
        if (typeof url === "object") {
            options = url;
            url = options.url || window.location.href;
        }
        
        options = options || {};
        if (!options.url) {
            options.url = url;
        }
        
        // Extract url parts.
        var matches = url.match(/^(https?):\/\/(ma(?:\.[a-z0-9]+\.ece[a-z]+)?\.rts\.ch)/);
        if (matches) {
            var endpoint = matches[0], scheme = matches[1], domain = matches[2];
            
            // If we are cross domain OR cross protocol.
            if (scheme != window.location.scheme || domain != window.location.hostname) {
                // Only then
                var s = $.ajaxSetup({
                    success: Function.prototype,
                    error: Function.prototype,
                    complete: Function.prototype
                }, options);

                // Get calls can be handled with jsonp
                if (s.type.toLowerCase() != "get" || s.dataType != "jsonp") {
                    // Create xhr object.
                    var xhr = (function(uid) {
                        var responseHeaders;
                        
                        return {
                            readyState: 0,
                            responseHeadersString: null,
                            setRequestHeader: function(name, value) {
                                throw new Error("Can't set request headers on a CORS ajax request.");
                            },
                            getAllResponseHeaders: function() {
                                return this.responseHeadersString;
                            },
                            getResponseHeader: function(key) {
                                var match;
                                if (this.responseHeadersString) {
                                    if (!responseHeaders) {
                                        responseHeaders = {};
                                        while((match = rheaders.exec(this.responseHeadersString))) {
                                            responseHeaders[match[1].toLowerCase()] = match[2];
                                        }
                                    }
                                    match = responseHeaders[key.toLowerCase()];
                                }
                                return match === undefined ? null : match;
                            },
                            abort: function(statusText) {
                                getRpc(endpoint).jqAjaxAbort(uid);
                                return this;
                            }
                        }
                    })(++uid);
                    
                    // Perform remote call.
                    getRpc(endpoint).jqAjax(
                        uid,
                        options,
                        function(response) {
                            var data = response[0], textStatus = response[1], jqXHR = $.extend(xhr, response[2]);
                            
                            s.success(data, textStatus, jqXHR);
                            s.complete(jqXHR, textStatus);
                        },
                        function(response) {
                            var args = response.message,
                                jqXHR = $.extend(xhr, args[0]), textStatus = args[1], errorThrown = args[2];
                            
                            s.error(jqXHR, textStatus, errorThrown);
                            s.complete(jqXHR, textStatus);
                        }
                    );
                    
                    // Return fake xhr object.
                    return xhr;
                }
            }
        }
        
        return ajx(options);
    };

    $.extend({ajax: jQueryAjaXDM});
});
