RtsDefine(["jquery", "underscore"], function($) {

/** 
 * @fileoverview This file contains RTSmodel implementation
 * @copyright RTS.ch
 * @version 0.9 
 
 */

//create the namespace for the mvc

rtsmvc = {};

(function(ns) {
	//use template syntax mustache.js
	_.templateSettings = {
  		interpolate : /\{\{(.+?)\}\}/g
	};

	//add basic string cutting function
	_.cut = function(str, maxLength, end){
		if(!_.isString(str) || maxLength <= 0) return;

		end = end || ' ...';

		return str.length <= maxLength ? str : str.substring(0, maxLength - end.length) + end;
	}

	//duration tag for template
	_.rtsDuration = function(secTime){
		var time = ''; var tf=0; var s = 0; var timeI = 2;

		if (secTime <= 0 || secTime == '') {
			return "--:--";
		}
		
		while(timeI >= 0){
			s = Math.pow(60, timeI);
			
			tf = Math.floor(secTime / s);

			if(tf > 0){
				time += (time == '' ? '' : ':' ) + ( tf < 10 ? '0'+tf.toFixed(0) : ''+tf.toFixed(0) );
				secTime -= tf * s;
			}

			timeI --;
		}
		if (time.length <= 2) {
			time ="00:"+time;
		}
		return time;
	}
	//date iso to rts convertion, use in template
	_.rtsISOtoTS = function (isoDate) {
		var date = 0;
		
		if (null !== isoDate) {
			var dateTime = isoDate.split(" ");
			if (dateTime.length == 2) {
				var parts = dateTime[0].match(/(\d+)/g);
				var date = new Date(parts[0], parts[1]-1, parts[2]).getTime();
			}
		}
		
		return date;
	}

	// Calculate time difference as Facebook does.
	_.sinceWhen = function (when, now) {
		// create some Date objects
		
		var now  = Math.round(new Date(now).getTime() / 1000);
		var when = Math.round(new Date(when).getTime() / 1000);

        // It returns the time difference in Seconds...
        var time_differnce = now-when;

		// To Calculate the time difference in Years...
		var years   = 60*60*24*365;
		
		// To Calculate the time difference in Months...
		var months  = 60*60*24*30;
		
		// To Calculate the time difference in Days...
		var days    = 60*60*24;
		
		// To Calculate the time difference in Hours...
		var hours   = 60*60;
		
		// To Calculate the time difference in Minutes...
		var minutes = 60;

        if((time_differnce/days) > 6) {
            var d = new Date(when*1000);
            return "le "+d.getDate()+"."+(d.getMonth()+1)+"."+d.getFullYear();
        }
        else if ((time_differnce/days) >= 1 && (time_differnce/days) <=6) {
        	if (Math.round(time_differnce/days) <= 1) {
        		return "il y a "+Math.round(time_differnce/days)+" jour";	
        	} else
            return "il y a "+Math.round(time_differnce/days)+" jours";
        }
        else if ((time_differnce/hours) > 1) {
            return "il y a "+Math.round(time_differnce/hours)+" heures";
        }
        else if ((time_differnce/hours) >= 1) {
            return "il y a "+Math.round(time_differnce/hours)+" heure";
        }
        else if ((time_differnce/minutes) > 1) {
            return "il y a "+Math.round(time_differnce/minutes)+" minutes";
        }
        else if ((time_differnce/minutes) <= 1 && (time_differnce) >= 10) {
            return "il y a quelques secondes ";
        }
        else {
            return "maintenant";
        }
	}
	//cut text and add button to show full text
	_.rtsMore = function(text, maxLength){
		var t = text.split(' '), tShort = [], tEnd = [], count = 0;

		for(var i = 0; i < t.length; i ++){
			count += t[i].length;
			
			if(count > maxLength){
				tEnd.p(t[i]);
			} else {
				tShort.p(t[i]);
			}
		}

		t = [];

		t.p('<p class="more">');
			t.p('<span class="short">')
				t.p(tShort.join(' '));
			t.p('</span>');

			t.p('<button> <span> + </span> </button>');

			t.p('<span class="end" style="display:none;">');
				t.p(tEnd.join(' ')); 
			t.p('</span>');
		t.p('</p>');
		
		return t.str();
	}

	// Helper function to correctly set up the prototype chain, for subclasses.
	// Similar to `goog.inherits`, but uses a hash of prototype properties and
	// class properties to be extended.
	var extend = function (protoProps, staticProps) {
		var parent = this, child;
		// The constructor function for the new subclass is either RtsDefined by you
		// (the "constructor" property in your `extend` definition), or defaulted
		// by us to simply call the parent's constructor.
		if (protoProps && _.has(protoProps, 'constructor')) {
		  child = protoProps.constructor;
		} else {
		  child = function (){ parent.apply(this, arguments); };
		}

		// Add static properties to the constructor function, if supplied.
		_.extend(child, parent, staticProps);

		// Set the prototype chain to inherit from `parent`, without calling
		// `parent`'s constructor function.
		var Surrogate = function(){ this.constructor = child; };
		Surrogate.prototype = parent.prototype;
		child.prototype = new Surrogate;

		// Add prototype properties (instance properties) to the subclass,
		// if supplied.
		if (protoProps) _.extend(child.prototype, protoProps);

		// Set a convenience property in case the parent's prototype is needed
		// later.
		child.__super__ = parent.prototype;
		
		child.prototype.clsid = _.uniqueId();
		
		return child;
	};

	// Backbone.Events
  	// ---------------

	// Regular expression used to split event strings
	var eventSplitter = /\s+/;

	// A module that can be mixed in to *any object* in order to provide it with
	// custom events. You may bind with `on` or remove with `off` callback functions
	// to an event; `trigger`-ing an event fires all callbacks in succession.
	//
	//     var object = {};
	//     _.extend(object, Backbone.Events);
	//     object.on('expand', function(){ alert('expanded'); });
	//     object.trigger('expand');
	//
	var Events = {

	// Bind one or more space separated events, `events`, to a `callback`
	// function. Passing `"all"` will bind the callback to all events fired.
	on: function(events, callback, context) {
	  var calls, event, list;
	  if (!callback) return this;

	  events = events.split(eventSplitter);
	  calls = this._callbacks || (this._callbacks = {});

	  while (event = events.shift()) {
	    list = calls[event] || (calls[event] = []);
	    list.push(callback, context);
	  }

	  return this;
	},

	// Remove one or many callbacks. If `context` is null, removes all callbacks
	// with that function. If `callback` is null, removes all callbacks for the
	// event. If `events` is null, removes all bound callbacks for all events.
	off: function(events, callback, context) {
	  var event, calls, list, i;

	  // No events, or removing *all* events.
	  if (!(calls = this._callbacks)) return this;
	  if (!(events || callback || context)) {
	    delete this._callbacks;
	    return this;
	  }

	  events = events ? events.split(eventSplitter) : _.keys(calls);

	  // Loop through the callback list, splicing where appropriate.
	  while (event = events.shift()) {
	    if (!(list = calls[event]) || !(callback || context)) {
	      delete calls[event];
	      continue;
	    }

	    for (i = list.length - 2; i >= 0; i -= 2) {
	      if (!(callback && list[i] !== callback || context && list[i + 1] !== context)) {
	        list.splice(i, 2);
	      }
	    }
	  }

	  return this;
	},

	// Trigger one or many events, firing all bound callbacks. Callbacks are
	// passed the same arguments as `trigger` is, apart from the event name
	// (unless you're listening on `"all"`, which will cause your callback to
	// receive the true name of the event as the first argument).
	trigger: function(events) {
	  var event, calls, list, i, length, args, all, rest;
	  if (!(calls = this._callbacks)) return this;

	  rest = [];
	  events = events.split(eventSplitter);

	  // Fill up `rest` with the callback arguments.  Since we're only copying
	  // the tail of `arguments`, a loop is much faster than Array#slice.
	  for (i = 1, length = arguments.length; i < length; i++) {
	    rest[i - 1] = arguments[i];
	  }

	  // For each event, walk through the list of callbacks twice, first to
	  // trigger the event, then to trigger any `"all"` callbacks.
	  while (event = events.shift()) {
	    // Copy callback lists to prevent modification.
	    if (all = calls.all) all = all.slice();
	    if (list = calls[event]) list = list.slice();

	    // Execute event callbacks.
	    if (list) {
	      for (i = 0, length = list.length; i < length; i += 2) {
	        list[i].apply(list[i + 1] || this, rest);
	      }
	    }

	    // Execute "all" callbacks.
	    if (all) {
	      args = [event].concat(rest);
	      for (i = 0, length = all.length; i < length; i += 2) {
	        all[i].apply(all[i + 1] || this, args);
	      }
	    }
	  }

	  return this;
	}

	};
	//get sub array of the object where controller events will be linked 
	var ControllerBinderGetSource = function(controller, ns){
			var c, cid;
			cid = controller.clsid;

			c = this._callbacks = this._callbacks || {};

			ns = ns || '_';

			if(!c[cid]) c[cid] = {};
			if(!c[cid][ns]) c[cid][ns] = [];

			return c[cid][ns];
		}
	//simple binder for communication beetwen controller and other objects
	var ControllerBinder = {
		//add controller to object (collection or Model)
		onController:function(controller, namespace){
			if(!controller) return this;
			var s = ControllerBinderGetSource.call(this,controller, namespace);


			//check if controller is already ref
			for(var i = 0; i < s.length; i ++){
			

				if(s[i] == controller){
					return this;
				}
			}

			s.push(controller);

			return this;
		},
		//remove controller from a object (collection or Model)
		offController:function(controller, namespace){
			var c = ControllerBinderGetSource.call(this,controller, namespace);


			for(var i = 0; i < c.length; i ++){
				if(c[i] === controller){
					delete c[i];
				}
			}

			return this;
		},
		//iteration over each controller and run disptacher callback
		triggerControllers:function(dispatcher){
		
			var c = this._callbacks || {};

			for(var cid in c){
				for(var ns in c[cid]){
					for(var i = 0; i < c[cid][ns].length; i ++){

						if(c[cid][ns][i]) dispatcher(c[cid][ns][i], ns);
					}
				}
			}

			return this;

		}
	};
	//global ver use to sync local and server date
	ns.ServerDate = new Date();
	
	//array with liste of all css loaded on page see function loadcss
	var cssLoaded = [];

	/**
	* 
	* @namespace Util
	*/
	var Util = {
		logMe : function(actionWhenLogged){
			ns.SessionMember().getMember(function(logged){
				if(!logged){
		            ns.Util.basicOpenPopup('https://' + ns.Util.env.domains.sso + '/UI/Login', _.bind(function(){
		                ns.SessionMember().reload({silentReload:true});
		                //get member is call when member is loaded.
		                ns.SessionMember().getMember(function(logged){
		                    if(logged) {

		                       actionWhenLogged.call(this);
							
		                       ns.SessionMember().triggerReload();
		                    }
		                    
		                }, this);
		            }, this));
	            } else {
	            	actionWhenLogged.call(this);
	            }
            }, this);
        },

		//base64 encoder and decoder use native function if is supported by browser
		Base64 : (function(){
			if(window.btoa && window.atob){
				//need proxy function btoa and atob, to avoid illegal invocation exception
				return {encode : function(x){return btoa(x)}, decode : function(x){return atob(x);}};
			} else {
				Base64 = {
					// private property
					_keyStr : "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=",

					// public method for encoding
					encode : function (input) {
					    var output = "";
					    var chr1, chr2, chr3, enc1, enc2, enc3, enc4;
					    var i = 0;

					    input = Base64._utf8_encode(input);

					    while (i < input.length) {

					        chr1 = input.charCodeAt(i++);
					        chr2 = input.charCodeAt(i++);
					        chr3 = input.charCodeAt(i++);

					        enc1 = chr1 >> 2;
					        enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
					        enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
					        enc4 = chr3 & 63;

					        if (isNaN(chr2)) {
					            enc3 = enc4 = 64;
					        } else if (isNaN(chr3)) {
					            enc4 = 64;
					        }

					        output = output +
					        Base64._keyStr.charAt(enc1) + Base64._keyStr.charAt(enc2) +
					        Base64._keyStr.charAt(enc3) + Base64._keyStr.charAt(enc4);

					    }

					    return output;
					},

					// public method for decoding
					decode : function (input) {
					    var output = "";
					    var chr1, chr2, chr3;
					    var enc1, enc2, enc3, enc4;
					    var i = 0;

					    input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");

					    while (i < input.length) {

					        enc1 = Base64._keyStr.indexOf(input.charAt(i++));
					        enc2 = Base64._keyStr.indexOf(input.charAt(i++));
					        enc3 = Base64._keyStr.indexOf(input.charAt(i++));
					        enc4 = Base64._keyStr.indexOf(input.charAt(i++));

					        chr1 = (enc1 << 2) | (enc2 >> 4);
					        chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
					        chr3 = ((enc3 & 3) << 6) | enc4;

					        output = output + String.fromCharCode(chr1);

					        if (enc3 != 64) {
					            output = output + String.fromCharCode(chr2);
					        }
					        if (enc4 != 64) {
					            output = output + String.fromCharCode(chr3);
					        }

					    }

					    output = Base64._utf8_decode(output);

					    return output;

					},

					// private method for UTF-8 encoding
					_utf8_encode : function (string) {
					    string = string.replace(/\r\n/g,"\n");
					    var utftext = "";

					    for (var n = 0; n < string.length; n++) {

					        var c = string.charCodeAt(n);

					        if (c < 128) {
					            utftext += String.fromCharCode(c);
					        }
					        else if((c > 127) && (c < 2048)) {
					            utftext += String.fromCharCode((c >> 6) | 192);
					            utftext += String.fromCharCode((c & 63) | 128);
					        }
					        else {
					            utftext += String.fromCharCode((c >> 12) | 224);
					            utftext += String.fromCharCode(((c >> 6) & 63) | 128);
					            utftext += String.fromCharCode((c & 63) | 128);
					        }

					    }

					    return utftext;
					},

					// private method for UTF-8 decoding
					_utf8_decode : function (utftext) {
					    var string = "";
					    var i = 0;
					    var c = c1 = c2 = 0;

					    while ( i < utftext.length ) {

					        c = utftext.charCodeAt(i);

					        if (c < 128) {
					            string += String.fromCharCode(c);
					            i++;
					        }
					        else if((c > 191) && (c < 224)) {
					            c2 = utftext.charCodeAt(i+1);
					            string += String.fromCharCode(((c & 31) << 6) | (c2 & 63));
					            i += 2;
					        }
					        else {
					            c2 = utftext.charCodeAt(i+1);
					            c3 = utftext.charCodeAt(i+2);
					            string += String.fromCharCode(((c & 15) << 12) | ((c2 & 63) << 6) | (c3 & 63));
					            i += 3;
					        }

					    }
					    return string;
					}
				}

				return {encode: Base64.encode, decode:Base64.decode};
			}
		})(),
		//absolute url for marts domaine
		domains: {
			marts: (function() {
				var matches = window.location.host.match(/([^.]+)\.ecedev\./);
				return "ma." + (matches ? matches[1] + ".ecedev." : "") + "rts.ch";
			})()
		},
			
		/**
		* make a mvc constructor shareable (singelton)
		* @memberof Util
		* @param {function} constructor the constructor of instance whitch will be shared
		* @param {function} onCreate callback for some action when shareble instance is create, format ((object) instance) -> undefined
		*/
		makeShareable : function(constructor, onCreate){
			//get free namespace
			var ns = _.uniqueId('__shareableNameSpace__');

			onCreate = onCreate || $.noop;

			return function(){
				return window[ns] || (function(){
					window[ns] = new constructor();
					onCreate(window[ns]);
					return window[ns];
				})()
			};
		},

		trace : function(){
          Util.env.trace.apply(window, Array.prototype.slice.apply(arguments));
		},

		/**
		* Returns an object with attributes that RtsDefine the environnement
		*/
		env: $.env,
		

	    /**
		 * Open a pop-up with reload of session member
		 * 
		 * @param {string} url Url of the new window to pop-up.
		 * @param {function} onCloseFn a function to call when the pop-up closes
		 * @param {object} attributes Attributes for the new pop-up.
		 */
		openPopup : function(url, onCloseFn, attributes) {
			if (arguments.length == 2 && !$.isFunction(onCloseFn)) {
				attributes = onCloseFn;
				onCloseFn = undefined;
			}
			
			var closeCb = onCloseFn || $.noop;

			onCloseFn = function(){
				if (ns.SessionMember) {
					ns.SessionMember().getMember(function(logged, member) {

						if (!logged) ns.SessionMember().reload();
					}, this);
				}

				closeCb();
			}
			
			this.basicOpenPopup(url, onCloseFn, attributes);
		},
		/**
		 * Open a pop-up
		 * 
		 * @param {string} url Url of the new window to pop-up.
		 * @param {function} onCloseFn a function to call when the pop-up closes
		 * @param {object} attributes Attributes for the new pop-up.
		 */
		basicOpenPopup : function(url, onCloseFn, attributes) {
			if (arguments.length == 2 && !$.isFunction(onCloseFn)) {
				attributes = onCloseFn;
				onCloseFn = undefined;
			}
			attributes = attributes || {};
			var popupname = attributes.name ? attributes.name : "marts_popup";
			// Prepare attributes.
			attributes = (function(attr) {
				var tmp = [];
				for (var key in attr) {
					tmp.push(key + "=" + (attr[key] === true ? "yes" : attr[key] === false ? "no" : attr[key]));
				}
				return tmp.join(",");
			})($.extend({
				width : 775,
				height : 520,
				scrollbars : true
			}, (attributes || {})));
			
			// Add consumer id.
			if ($.env.consumer) {
				url += (url.indexOf("?") != -1 ? "&" : "?") + "cid=" + $.env.consumer.id;
			}

			// Open window.
			var popUp = window.open(url, popupname, attributes);
			if(popUp.focus) popUp.focus();
			var checker = function() {
				if (popUp.closed) {

					
					if (onCloseFn)
						onCloseFn();
				} else {
					setTimeout(checker, 300);
				}
			};
		  	checker();	  
		},
		//basic  cookie creation 
		createCookie : function (name, value, days) {
		    if (days) {
		        var date = new Date();
		        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
		        var expires = "; expires=" + date.toGMTString();
		    }
		    else var expires = "";
		    document.cookie = name + "=" + value + expires + "; path=/";
		},

		//reading cookie
		getCookie : function (c_name) {
		    if (document.cookie.length > 0) {
		        c_start = document.cookie.indexOf(c_name + "=");
		        if (c_start != -1) {
		            c_start = c_start + c_name.length + 1;
		            c_end = document.cookie.indexOf(";", c_start);
		            if (c_end == -1) {
		                c_end = document.cookie.length;
		            }
		            return unescape(document.cookie.substring(c_start, c_end));
		        }
		    }
		    return null;
		},
		//get param from url
		getURLParameter: function (name) {
		    return decodeURIComponent((new RegExp('[?|&]' + name + '=' + '([^&;]+?)(&|#|;|$)').exec(location.search)||[,""])[1].replace(/\+/g, '%20'))||null;
		},
		
		/**
		 * Transforms a key value attribute map into html attributes.
		 * 
		 * @param {Object} attributes
		 * @return {String}
		 */
		attrToHtml: function(attributes) {
			
			var tmp = [];
			for (var name in attributes) {
				tmp.push(name + '="' + _.escape(attributes[name]) + '"');
			}
			
			return tmp.join(" ");
		},
		
		/**
		 * load css by javascript
		 * 
		 * @memberOf Util
		 * @param {string} url path to the css.
		 */
		loadCss : function (url) {
			
			for(var i = 0; i < cssLoaded.length; i++){
				if(cssLoaded[i] == url) return;
			}
			cssLoaded.push(url);

			var link = document.createElement("link");
			link.type = "text/css";
			link.rel = "stylesheet";
			link.href = url;
			document.getElementsByTagName("head")[0].appendChild(link);
			
		}
	};
	//simulate ajax request
	var fakeAjax = function(options){
		if(_.isFunction(options.success)){
			options.success.call(this, {}, "200", {});
		}
	};
	//merge two function in one if source function is RtsDefine
	var mergeFnIfDef = function(object, key, callback) {
		
		// Normal execution flow.
		if (object[key]) {
			object[key] = mergeFunctions.call(this, callback, object[key]);
		} else {
			object[key] = callback;
		}
		
		// Hook
		var hook = "pre" + key.charAt(0).toUpperCase() + key.substr(1) + "Hook";
		if (object[hook]) {
			object[key] = mergeFunctions.call(this, object[hook], object[key]);
		}

		return object;
	}
	//send model attribute modification to server
	var syncAttribute = function( key, val, options ){
		if(this.hasId()){
			
			options = options || {};
			options.data = this.keyValToObject(key, val);
			options.url = urlParam(this.getUrl(), this.keyValToObject(this.idAttribute, this.attributes[this.idAttribute]));
			this.requestPUT(options);
		}
	};
	//send model attribute modification to server, by uri
	var syncVirtualAttribute = function( key, val, options ){
		if(this.hasId()){
			options = options || {};
			options.data = this.keyValToObject(key, val);

			options.url = urlParam(this.getUrl()+"/"+key, this.keyValToObject(this.idAttribute, this.attributes[this.idAttribute]));
			this.requestPUT(options);
		}
	};
	//simple bind context to callback function
	var bindAjaxCallback = function(callback, bind){
		if(callback){
			return function(){ callback.apply(bind, arguments);}
		}
		return undefined;
	};
	//merge 2 function in 1
	var mergeFunctions = function(functionA, functionB){
		var bind = this;
		return function(){
			functionA.apply(bind, arguments);
			functionB.apply(bind, arguments);
		};
	};
	//replace url param with vars in params object
	var urlParam = function(url, params){
		//check if we have object to replace in string
		if(_.isObject(params)) {
			//if yes replace
			_.each(params, function(value, key) {
				url = url.replace(":"+key, _.isUnRtsDefined(value) ? '' : value);
			});
		}
		//remove all :* params
		return url.replace(/:[^\/]+[\/$]*/g, '');


	};
	//entry point for each ajax request made by the mvc
	var requestLauncher = function(options){
		options = options || {};

		mergeFnIfDef.call(this, options, "success", function(data, textStatus, jqXhr){
			
			var gmtStrDate = jqXhr.getResponseHeader("Date");

			if(gmtStrDate){
				gmtData = new Date(gmtStrDate);
				if(!isNaN(gmtData.getTime())){
					ns.ServerDate = gmtData;
				}
			}
			
		});

		options.success = bindAjaxCallback(options.success, this) || $.noop;
		options.error =  bindAjaxCallback(options.error, this) || $.noop;
		options.dataType = 'json';
		
		if ($.env.consumer) {
			options.url += 
				(options.url.indexOf('?') != -1 ? '&' : '?') + 
				"cid=" + $.env.consumer.id;
		}
		
		options.url = "https"  + "://" + $.env.domains.marts + options.url;
		
		$.ajax(options);
	};
	//add default value for each attribute in model
	var setDefaultAttr = function(){
		var model = this;
		var defaultHandler = null;
		var value;

		for(var key in this.defAttr){
			value = this.defAttr[key];

			defaultHandler = value['default'] || null;


			if(_.isFunction(defaultHandler)){
				model.attributes[key] = defaultHandler.call(model, key);
			} else if (_.isString(defaultHandler)){
				model.attributes[key] = defaultHandler;
			}
		}

	}
	//get url collection RtsDefine by parent attribute and idAttribute
	var collURL = function(){
		//if link with model
		if(this.parent){
			return this.parent.urlWithId() +'/'+ this.parentAttribute +'/';

		} else if(this.id && this.idAttribute){
			var o = {};
			o[this.idAttribute] = this.id;
			
			return urlParam(_.isFunction(this.url) ? this.url() : this.url, o) + '/';
		}
		return this.url;
		
	}
	//set a attribute to model and trigger callbacks
	var setModelAttribute = function(key, val, noAttrEvent){
		//no change find, cancel merge
		if(this.attributes[key] === val) return;

		var old = this.attributes[key];

		if(old === val) return old;

		if(old instanceof Collection){
			old.applyMeta(val);
			val = old;
		} else {
			this.attributes[key] = val;
		}

		if(!noAttrEvent){
			this.triggerControllers(function(controller, namespace){
				controller.dispatchModel(namespace, 'change',key, val, old);
				
			});
		}

		return old;

	}
	//RtsDefine value type is serializable
	var isExportable = function(value){
		return _.isString(value) || _.isNumber(value) || _.isBoolean(value) || _.isDate(value);
	}
	//export only string, number, boolean
	var exportAttributes = function(attr){
		var rts  = {};

		for(key in attr){
			
			if( isExportable(attr[key]) ){
				rts[key] = attr[key];
			}
		}
		return rts;
	}
	//trigger event when somthing change in model
	var onChangeStar = function(){

		this.triggerControllers(function(controller, namespace){
			controller.dispatchModel(namespace, 'change', '*');
		});
	}
	//trigger event when model is remove
	var onRemoveModel = function(){
		this.triggerControllers(function(c, namespace){
			c.dispatchModel(namespace, 'model', 'remove');
		});
	}
	//trigger inline set callback RtsDefine directly in model
	var inlineOnSetModel = function(key, value){
		if(this.defAttr[key]){
			if(_.isFunction(this.defAttr[key][MODEL_ON_SET])){
				this.defAttr[key][MODEL_ON_SET].call(this, value);
			}
		}
	}

	//the core of the model. Model is has two side. The core and the shell. The core containe data and function. The shell is proxy that call all core methods, expect for loadById with is a little custom.
	var ModelCore = function(){
		this.shell = null;
		this.attributes =  {};
		this.idAttribute = this.idAttribute || null;
		this.url = this.url || null;
		

		setDefaultAttr.call(this);

		this.init();
	};
	//add inherence possibility to model
	
	ModelCore.extend = extend;

	var MODEL_DEF_ATTR_SYNC     = "sync";
	var MODEL_DEF_ATTR_VAL_FAIL = "onValidateFailed";
	var MODEL_VALIDATION        = "validation";
	var MODEL_ON_SET            = "onSet";
	var MODEL_DEF_URI_SYNC	 	= "uriSync";
	var MODEL_LIVE_SYNC_METHOD	= "attrSync"

	/**
     * Model prototype
     */
    _.extend(ModelCore.prototype, ControllerBinder, {
		
		/**
		 * Initialize
		 * @memberOf Model#
		 * @example

var M = rtsmvc.Model.extend({
	init : function(){
		this.attributes["hello"] = "world";
	}
});
		 */
		init : function(){
			//empty need to inherite
		},
		
		/**
	     * Check if there is an id
	     * @return {boolean} True if attribute exists, else false 
	     * @memberOf Model#
	     * @example
var myModel = new rtsmvc.Model();

myModel.hasId() // return fales;

myModel.loadById(12, {success:function(){
	this.hasId(); // return ture;
}});
	     */
		hasId : function(){
			return this.idAttribute && this.attributes[this.idAttribute] !== undefined;
		},
		/**
		* return url of current model
		* @memberOf Model#
		*/
		getUrl : function(){
			return _.isFunction(this.url) ? this.url() : this.url;
		},
		merge : function(attrs){

			for(name in attrs){
				if(this.attributes[name] instanceof Collection){
					this.attributes[name].applyMeta(attrs[name]);
				} else {
					this.attributes[name] = attrs[name];
					inlineOnSetModel.call(this, name, attrs[name]);
				}
			}
		},
		/**
		 * Set attributes
		 * @param {object} key attribute name
		 * @param {object} value attribute value
		 * @param {object} options maybe callback informations
		 * @memberOf Model#
		 */
		set : function(key, value, options){
			//set first param is object
			if(_.isObject(key)){
				var me = this;
				_.each(key, function(v,k){
					me.set(k, v, options);
				});
				return;
			}

			if(key == this.idAttribute){
				throw("you cannot set attributes "+key+" because it is id of this model");
				return;
			}

			options = options || {};
			
			var old = setModelAttribute.call(this, key, value);
			
			if(old !== value){
				onChangeStar.call(this);
			}

			if(!options.noLiveSync && this.defAttr && _.isObject(this.defAttr[key]) ){
				if( this.hasId() && this.validate(key) ){

					if( _.isFunction(this.defAttr[key][MODEL_DEF_ATTR_SYNC]) ){
						this.defAttr[key][MODEL_DEF_ATTR_SYNC].call(this, key, value, options);
					} 
					else {
						if (_.isBoolean(this.defAttr[key][MODEL_DEF_URI_SYNC])) {
							syncVirtualAttribute.call(this, key, value, options);
						}
						else {
							
							syncAttribute.call(this, key, value, options);
						}
					}
				}
			}

			inlineOnSetModel.call(this, key, value);

			
		},
		reload : function(options){
			options = options || {};
			var id = this.getId();
			
			this.attributes = {};

			//ModelCache.remove(this);

			if(this.loaded) delete this.loaded;

			setDefaultAttr.call(this);


			if(!options.silentReload){
				mergeFnIfDef.call(this, options, "success", function(){
					this.triggerControllers(function(c, n){
						c.dispatchModel(n, "model", "reload");
					});
				});
			}
			this.loadById(id, options);

			



		},
		triggerReload : function(){
			this.triggerControllers(function(c, n){
				c.dispatchModel(n, "model", "reload");
			});
		},
		/**
		 * Set local attribute
		 * @param {object} key attribute name
		 * @param {object} value attribute value
		 * @param {object} options maybe callback informations
		 * @memberOf Model#
		 * @example
var media = new ModelMedia();
media.setLocal("title", "hello world"); // no call to server
media.set("title", "hello world"); //call to server
		 */
		setLocal : function(key, value, options){
			options = options || {};
			options.noLiveSync = true;
			this.set(key, value, options);
		},

		/**
		 * Get id
		 * @return {int | undefined} Return object id 
		 * @memberOf Model#
		 */
		getId : function(){
			return this.hasId() ? this.attributes[this.idAttribute] : undefined;
		},
		/**
		* return true if attribute is undefined
		* @param {string} key attribute name
		* @return {boolean} true if RtsDefined otherwise false
		* @memberOf Model#
		**/
		isUnRtsDefinedAttr : function(key){
			if(this.defAttr[key]){
				return _.isUnRtsDefined(this.attributes[key]);
			}
			//return true not RtsDefined
			return true;
		},
		/**
		 * Get value of a particular attribute
		 * @param {object} key attribute name
		 * @return {object | undefined} Attribute value
		 * @memberOf Model#
		 * @example
var myModel = new ModelMedia();
myModel.loadById(12, {success : function(){
	this.get("title"); // return title of model
	this.get("notRtsDefinedfield") // throw exception
}});
		 */
		get: function(key) {
			if( this.defAttr[key] ) {
				return this.attributes[key];
			}
			throw("attributes "+key +" not found");
		},
		
		/**
		 * Validate an attribute
		 * @param {object} AttrName attribute name
		 * @return {boolean} True if Attribute is valide, else false
		 * @memberOf Model#
		 */
		validate : function(attrName) {
			var val = this.attributes[attrName] || null;
			if(this.defAttr && _.isObject(this.defAttr[attrName]) && _.isArray(this.defAttr[attrName][MODEL_VALIDATION])){
				for(var i = 0; i < this.defAttr[attrName][MODEL_VALIDATION].length; i ++){
					var validator = this.defAttr[attrName][MODEL_VALIDATION][i];
					if( _.isFunction(Model.validators[validator]) ){
						if(!Model.validators[validator](val, attrName, this)) {
							return false;
						}
					} else {
						throw("validator :" + validator + " , not found");
					}
				}
			}
			return true;
		},
		
		/**
		 *	Validate all model attributes
		 *	@return {boolean} True if all attributes are valid, else false
		 * @memberOf Model#
		 */
		validateAll : function() {
			var isAllOk = true;
			var model = this;
			_.each(this.defAttr, function(value, key) {
				if(!model.validate(key) && isAllOk){
					isAllOk = false;
				}
			});
			return isAllOk;
		},
		
		/**
		 * @param {string} url Url
		 * @param {array} params Url parameters
		 * @return {string} A modified url
		 * @memberOf Model#
		 */
		urlParam : function(url, params){
			//proxy method
			return urlParam.call(this, url, params);
		},
		
		/**
		 * @return {string}
		 * @memberOf Model#
		 * @example
var ModelMedia = rtsmvc.Model.extend({
	url:'/source/to/my/model/:idMedia',
	idAttribute:'idMedia'
});

var myModel = new ModelMedia();

myModel.urlWithId(); //return /source/to/my/model/

myModel.loadById(12, {success:function(){
	this.urlWithId() //return /source/to/my/model/12
}});
		 */
		urlWithId : function(){
			return urlParam.call(this, this.getUrl(), this.keyValToObject(this.idAttribute, this.getId()));
		},
		
		/**
		 * Convert a <key;val> to an object
		 * @param {object} key Attribute key
		 * @param {object} val Attribue value
		 * @returns A object
		 * @type {object}
		 * @memberOf Model#
		 */
		keyValToObject : function(key, val){
			var o = {};
			o[key] = val;
			return o;
		},
		
		/**
		 * Load playlist by id
		 * Method: GET
		 * @param {int} id Playlist Id
		 * @param {object} options options
		 * @memberOf Model#
		 * @example
var myModel = new ModelMedia();

myModel.loadById(12, {success : function(){
	//my model is load
	//if model is link with a controller this action launch
	// events change:*, change:[attribute name]
}});

!Important all Model have a transparent local cache, that means if you load 2 models with the same id. Only the first call made a ajax request
		 */
		loadById:function(id, options) {
			options = options || {};
			options.url = urlParam(this.getUrl(), this.keyValToObject(this.idAttribute, id));
			
			

			this.requestGET(options);
		},
		applyWaitingSuccess : function(){
			this.loaded = true;
				
			var cb = this.__waitingCb || [];

			for(var i = 0; i < cb.length; i++){
				cb[i].apply(this, arguments);
			}
		},
		/**
		 * Save Model
		 * Method: PUT
		 * @param {object} options options
		 * @memberOf Model#
		 */
		save : function(options){
			if(this.hasId()){
				options = options || {};
				options.url = urlParam(this.getUrl(), this.keyValToObject(this.idAttribute, this.attributes[this.idAttribute]));
				options.data = exportAttributes(this.attributes);
				this.requestPUT(options);

			} else {
				throw("no id found, object cannot be saved, use create method instead");
			}
		},
		
		/**
		 * Remove from playlist
		 * Method: remove
		 * @param {object} options options
		 * @memberOf Model#
		 */		
		remove : function(options){
			if(this.hasId()){
				options = options || {};
				options.url = urlParam(this.getUrl(), this.keyValToObject(this.idAttribute, this.attributes[this.idAttribute]));
				this.requestDELETE(options);
			} else {
				throw("no id found, object cannot be delete");
			}
		},

		/**
		 * Make a GET request
		 * @param {object} options options
		 * @memberOf Model#
		 */
		create : function(options){
			if(!this.hasId()){
				options = options || {};
				options.url = urlParam(this.getUrl());
				options.data = exportAttributes(this.attributes);

				this.requestPOST(options);
			} else {
				throw("you cannot create object with an id");
			}
		},
		
		/**
		 * Make a GET request
		 * @param {object} options options
		 * @memberOf Model#
		 */
		requestGET : function(options) {
			//if id RtsDefine throw error
			options = options || {};
			options.type = 'GET';
			
			mergeFnIfDef.call(this, options, "success", this.mergeAttributes);
			
			requestLauncher.call(this, options);
		},
		
		/**
		 * Make a PUT request
		 * @param {object} options options
		 * @memberOf Model#
		 */
		requestPUT : function(options){
			options = options || {};
			options.type = 'PUT';

			mergeFnIfDef.call(this, options, "success", this.mergeAttributes);
			

			requestLauncher.call(this, options);
		},
		
		/**
		 * Make a POST request
		 * @param {object} options options
		 * @memberOf Model#
		 */
		requestPOST : function(options){
			options = options || {};
			options.type = 'POST';
			

			mergeFnIfDef.call(this, options, "success", this.mergeAttributes);
			

			requestLauncher.call(this, options);
		},
		
		/**
		 * Make a DELETE request
		 * @param {object} options options
		 * @memberOf Model#
		 */
		requestDELETE : function(options){
			options = options || {};
			options.type = 'DELETE';

			mergeFnIfDef.call(this, options, "success", this.resetAttributes);
			
			requestLauncher.call(this, options);
		},
		
		/**
		 * Merge attributes
		 * @param {object} data data
		 * @param {string} textStatus status
		 * @param {object} jqXHR xhr
		 * @memberOf Model#
		 */
		mergeAttributes : function(data, textStatus, jqXHR) {
			
			for(var key in data){
				setModelAttribute.call(this, key, data[key], true);
			}
			for(var key in data){
				inlineOnSetModel.call(this, key, data[key]);
			}

			this.triggerControllers(function(controller, namespace){
				for(var key in data){
					controller.dispatchModel(namespace, 'change',key, data[key], null);
				}
			});
			this.applyWaitingSuccess();
			onChangeStar.call(this);
			this.readyTrigger();
			
		},
		
		/**
		 * Empty all attributes
		 */
		resetAttributes : function() {
			this.attributes = {};

			
			onChangeStar.call(this);
			onRemoveModel.call(this);
		},
		
		/**
		 * Todo
		 */
		requestError : function(json){
			
		},
		readyTrigger : function(controllerFilter){
			if(this.hasId()){
				//launch trigger her

				this.triggerControllers(function(c, n){
					if(
						(controllerFilter && controllerFilter == c) ||
						!controllerFilter
					) {

						c.dispatchModel(n, "model", "ready");
					}
				});
			}
		}

	});
	

	
	var ModelCache = {
		cache : {},
		findOrStore : function(core, id){
			var type = this.getType(core._typeuid);

			return type[id] || (type[id] = core);
		},
		store : function(data, constructor, idAttribute){
			var id = data[idAttribute];
			var type = this.getType(constructor._typeuid);

			if(!type[id]){
				type[id] = new constructor();
				type[id].attributes = data;
				type[id].applyWaitingSuccess();
			} else {
				//replace
				type[id].mergeAttributes(data);
			}

		},
		getType : function(uid){
			return  this.cache[uid] || (this.cache[uid] = {});
		},
		remove : function(core){
			var type = this.getType(core._typeuid);

			if(type[core.getId()]){
				delete type[core.getId()];
			}
		}
	};

	/**
	 * Construction of Model
	 * @constructor
	 * @global
	 * @desc Model construction function. The construction flow is: 1. RtsDefine attributes object. 2. apply default attributes values. 3. call function init.
	 */
	var Model = function(){


	};

	Model.extend = function(protoProps, staticProps){
		var staticProps = staticProps || {};
		protoProps._typeuid = _.uniqueId();

		var coreConstructor = ModelCore.extend.call(ModelCore, protoProps, staticProps);
		coreConstructor._typeuid = protoProps._typeuid;

		var shellConstructor = function(){
			this.core = new coreConstructor();
			this.core.shell = this;
		};

		shellConstructor.createCore = function(data){
			ModelCache.store(data, coreConstructor, protoProps.idAttribute);
		};

		//add properties from model
		_.extend(shellConstructor.prototype, Model.prototype);
		if(staticProps) _.extend(shellConstructor, staticProps);
		//add new properties from core
		for(key in protoProps){

			if(_.isFunction(protoProps[key])){
				shellConstructor.prototype[key] = ((function(key){

					return function(){
						return coreConstructor.prototype[key].apply(this.core, arguments);
					};

				})(key));
			}

		}

		return shellConstructor;
		
	};


	//apply binding from shell to core
	for(key in ModelCore.prototype){


		Model.prototype[key] = ((function(key){

			return function(){
				return ModelCore.prototype[key].apply(this.core, arguments);
			};

		})(key));
	}
	//change core function loadById

	Model.prototype.loadById = function(id, options){
		
		if(this.core.hasId()) return;
		options = options || {};



		var oldCore = this.core;
		this.core = ModelCache.findOrStore(this.core, id);

		//if core change get callback of old core and copy them into the core
		if(oldCore !== this.core){
			var c, oc;

			c = this.core._callbacks || (this.core._callbacks={});
			oc = oldCore._callbacks;

			
			for(var cid in oc){
				for(var ns in oc[cid]){
					if(!c[cid]) c[cid] = {};
					if(!c[cid][ns]) c[cid][ns] = [];
					_.extend(c[cid][ns], oc[cid][ns]);
				}
			}


			//copy core callbacks
			//this.core.__waitingCb = oldCore.__waitingCb;

			//fake success callback if needed
			//fake event load
			if(this.core.loaded){

				if(options.reload){
					this.core.reload(options);
				} else {

					if(options && options.success) options.success.call(this.core, this.core, "success", {});

					this.core.readyTrigger();
				}
			} else {
				//not yet loaded but another shell waiting for request response add to wait liste
				var cb = this.core.__waitingCb || (this.core.__waitingCb = []);
				var m = this;
				
				cb.push(function(){
					if(options && options.success) options.success.call(m.core, m.core, "success", {});

					m.core.readyTrigger();
				})
				
			}
			

		} else {
			
			//same core object not loaded
			
			return this.core.loadById.apply(this.core, arguments);
		}

		
	};



	/**
	 * Model Validator
	 * @namespace Model.validators
	 */

	Model.validators = {
		/**
		 * Check if an attribute is RtsRequired or not 
		 * @param {object} value value
		 * @param {string} key key
		 * @return {boolean} If value is null returns false, else true
		 * @memberof Model.validators
		 */
		RtsRequired:function(value, key){
			return !_.isNull(value);
		},
		
		/**
		 * @param {object} value value
		 * @param {string} key key
		 * @param {Model} modelInstance model instance
		 * @return {boolean} Returns true if email is valid, else false
		 * @memberof Model.validators
		 */
		email:function(value, key, modelInstance){

		},
		
		/**
		 * @param {object} value value
		 * @param {string} key key
		 * @param {Model} modelInstance Model Instance
		 * @return {boolean} Return true if password is valid, else false
		 * @memberof Model.validators
		 */
		password:function(value, key, modelInstance){
			return value == modelInstance.attributes[key+'_confirm'];
		}
	};
	

	var onLoadCollection = function(page, limit, context){
		
		
		return function(){
			context.triggerControllers(function(controller, namespace){
				controller.dispatchCollection(namespace, "load", context, page, limit);
			});
		};
	};

	var onChangeSizeCollection = function(newSize){
		var old = this.size;
		this.size = newSize;

		this.triggerControllers(function(controller, namespace){
			controller.dispatchCollection(namespace, "change:size", newSize, old);
		});

	};

	var onChangeMetaCollection = function(newMeta){
		var old = this.meta;
		this.meta = newMeta;


		//lauch events
		var me = this;

		_.each(newMeta, function(value, name){
			me.triggerControllers(function(controller, namespace){
				//collection:change:meta:video_count

				controller.dispatchCollection(namespace, "change:meta:"+name, value, name, old[name] || null);
			});
		});
	};

	var loadRef = function(refs){
		for(var key in refs){

			for(var i = 0; i < refs[key].length; i++){
				ns[key].createCore(refs[key][i]);
			}
		}
	};


	/**
	 * Collection
	 * @constructor
	 * @global
	 * @example
//linking collection with a model


var myModel = rstmvc.Model.extend({
	defAttr : {
		myCollection:{
			//use qote to avoid bug with ie

			'default' : function(){
				//first param is the referance to the model
				//second param is the name of the attribute
				return new MyCollection(this, 'myCollection');
			}
		}
	}
});

var myModelInstance = new myModel();
myModelInstance.loadById(12);

var coll = myModelInstance.get('myCollection');
//loading collection
coll.load(0, 100);

	 */
	var Collection = function(parent, parentAttribute){
		this.size = 0;
		this.meta = {};
		this.items = {};
		this.model = this.model || null;
		this.url = this.url || null;
		this.parent = parent || null;
		this.parentAttribute = parentAttribute || null;
	};


	_.extend(Collection.prototype, ControllerBinder, {
		/**
		 * pattern use for paginate 
		 * @type string
		 * @memberOf Collection#
		 */
		patternLimit:'page/:page/limite/:limit',
		patternItem:':id',

		applyMeta : function(meta){
			this.meta = meta.meta || {};
			this.size = meta.size || 0;
		},
		/**
		 * Get collection size
		 * @return {int} Return collection size
		 * @memberOf Collection#
		 */
		getSize : function(){
			return this.size;
		},
		
		/**
		 * Get local collection size
		 * @return {int} local collection size
		 * @memberOf Collection#
		 */
		getLocalSize : function(){

			return _.size(this.items);	
		},
		/**
		* Set id to orphan collection
		* @param{int} the id to set
		* @memberOf Collection#
		* @example 
var col = new MyCollection();
col.setId(123);
col.load(0,100);

		*/
		setId : function(id){
			this.id = id;

		},
		/**
		* Define order server side for collection
		* All items of the collection will be reset when order change
		* @memberOf Collection#
		* @param{string}
		*/
		setOrder : function(order){
			this.order = order;
			this.reset();
		},
		/**
		 * Load part or all element of the collection
		 * @param{int} page page
		 * @param{int} limit limit
		 * @param{object} options options
		 * @return {Collection} The needed part of the playlist
		 * @memberOf Collection#
		 * @example
var myCol = new MediasCollection();

myCol.load(0, 100); //load first page of 100 items
		 */
		load : function(page, limit, options){
			options = options || {};

			

			if(this.isInCache(page, limit)){
				fakeAjax.call(this, options);

				return;
			}
			options.url = collURL.call(this) + this.getLimitChunck(page, limit);
			if(this.order){
				options.url += '/order/'+this.order;
			}

			options.listLimit = [page, limit];

			mergeFnIfDef.call(this, options, "success", onLoadCollection(page, limit, this));


			this.requestGET(options);
		},
		isInCache : function(page, limit){
			
			var absStart = page * limit;
			var absEnd = (absStart + limit) - 1;
			var elCount = 0;
			var i = 0;

			for(var key in this.items){
				
				if(i >= absStart && i <= absEnd){
					elCount ++;
					if(elCount >= limit) return true;
				}

				i ++;
				
			}
			
			return false;
		},
		/**
		 * @param {object} key key
		 * @return {object} Return meta attribute associated to key
		 * @memberOf Collection#
		 */
		getMeta : function(key) {
			return this.meta[key];
		},

		/**
		 * Get an element at specific position
		 * @param{int} pos position
		 * @return {object} An element of the collection
		 * @memberOf Collection#
		 */
		getItem : function(pos) {
			return this.items[pos];
		},
		/**
		* callback iteration over all items in collection
		* @memberOf Collection#
		* @param {function} callback type of callback is (index, item)
		* @param {object} context context of the callback by default collection context
		*/
		each : function(callback, context){
			context = context || this;

			for(var key in this.items){
				callback.call(context, key, this.items[key]);
			}
		},
		/**
		* callacbk iteration over items in local offset
		* @memberOf Collection#
		* @param {number} page the page offset
		* @param {number} limit the limit offset
		* @param {function} callback type of callback is (index, item)
		* @param {object} context context of the callback by default collection context
		*/
		eachOffset : function(page, limit, callback, context){
			var i = 0;
			var absStart = page * limit;
			var absEnd = (absStart + limit) - 1;

			context = context || this;

			for(var key in this.items){
				//console.log((i >= absStart) +'- && -'+ (i <= absEnd));

				if(i >= absStart && i <= absEnd){
					callback.call(context, key, this.items[key]);
				}

				i ++;
			}

		},
		/**
		 * Get a chunck of a specific length
		 * @param {int} page start page
		 * @param {int} limit limit
		 * @return {string} Url parameters
		 * @memberOf Collection#
		 */
		getLimitChunck : function(page, limit) {
			return urlParam(this.patternLimit, {page:page, limit:limit});
		},
		
		/**
		 * Add an item at first position of list
		 * @param {item} item item to add
		 * @param {object} options options, options.local: add item at first position to the collection but dont request the server
		 * @memberOf Collection#
		 */
		addItemAtFirst : function(item, options) {
			options = options || {};
			options.url = collURL.call(this) + urlParam(this.patternItem, {id:item.getId()});
			options.itemToAdd = item;
			this.requestPOSTArticle(options);

			//launch event on controllers linked with this collection, event name collection:addItemAtFirst
			this.triggerControllers(function(controller, namespace){
				controller.dispatchCollection(namespace, "addItemAtFirst", item);
			});
		},
		
		/**
		 * Remove an item from collection
		 * @param {item} item item to remove
		 * @param {object} options options
		 * @memberOf Collection#
		 */
		removeItem : function(item, options){
			options = options || {};
			options.url = collURL.call(this) + urlParam(this.patternItem, {id:item.getId()});
			options.itemToRemove = item;
			this.requestDELETEArticle(options);

			//adjuste meta size
			this.size --;

			//launch event on controllers linked with this collection, event name collection:removeItem
			this.triggerControllers(function(controller, namespace){
				controller.dispatchCollection(namespace, "removeItem", item);
			});
		},
		
		/**
		 * Clear collection
		 * @param {object} options options
		 * @memberOf Collection#
		 */
		clear : function(options){
			options = options || {};
			options.url = collURL.call(this) + urlParam(this.patternItem);
			options.url = options.url.slice(0,options.url.length-1);
			this.requestDELETE(options);
		},
		/**
		* reset collection
		* @memberOf Collection#
		*/
		reset : function(){
			this.items = [];
			this.size = 0;
		},
		
		/**
		 * Make a DELETE request
		 * @param {object} options options
		 * @memberOf Collection#
		 */
		requestDELETE: function(options){
			options = options || {};
			options.type = 'DELETE';

			mergeFnIfDef.call(this, options, "success", this.resetItem);

		
			requestLauncher.call(this, options);
		},


		/**
		 * Make a POST request
		 * @param {object} options options
		 * @memberOf Collection#
		 */
		requestPOSTArticle : function(options){
			
			if(options.local){

				
				this.mergeItem(options.itemToAdd.shell, {addPosition:0});

				if(options.success){
					options.success({}, "fakeRequest", {});
				}
		
			} else {
				options = options || {};
				options.type = 'POST';

				var itemToAdd = options.itemToAdd;
				delete options.itemToAdd;
				
				var itemSync = function(data, textStatus, jqXHR){
					this.mergeItem(itemToAdd.shell, data, textStatus, jqXHR);
				};

				mergeFnIfDef.call(this, options, "success", itemSync);

				requestLauncher.call(this, options);
			}
		},


		/**
		 * Make a DELETE request
		 * @param {object} options options
		 * @memberOf Collection#
		 */		
		requestDELETEArticle : function(options){
			options = options || {};
			options.type = 'DELETE';
			var itemToRemove = options.itemToRemove || null;
			delete options.itemToRemove;
			
			var itemSync = function(data, textStatus, jqXHR){
				this.unmergeItem(itemToRemove, data, textStatus, jqXHR);
			};

			mergeFnIfDef.call(this, options, "success", itemSync);

			requestLauncher.call(this, options);
		},

		/**
		 * Make a GET request
		 * @param {object} options options
		 * @memberOf Collection#
		 */
		requestGET : function(options){
			options = options || {};
			options.type = 'GET';
			
			mergeFnIfDef.call(this, options, "success", function(data, textStatus, jqXHR){
				this.mergeCollection(options.listLimit, data, textStatus, jqXHR);

			});

			requestLauncher.call(this, options);
		},
		
		/**
		 * Merge Collection
		 * @param {int} limit limit
		 * @param {object} data data
		 * @param {textStatus} textStatus Status
		 * @param {object} jqXHR xhr
		 * @memberOf Collection#
		 */
		mergeCollection : function(limit, data, textStatus, jqXHR){
			if(data.ref){
				loadRef.call(this, data.ref);
			}
			
			

			onChangeSizeCollection.call(this, data.size);
			onChangeMetaCollection.call(this, data.meta);

			if(data.items !== null) {
				this.mergeList(data.items, limit);
			}
			
		},
		
		/**
		 * Merge List
		 * @param {object} itemsMerge items
		 * @param {int} limit limit
		 * @memberOf Collection#
		 */
		mergeList : function(itemsMerge, limit){
			var items = this.items;

			for(var i = 0; i < itemsMerge.length; i ++){
				var inst = new this.model();

				inst.core = ModelCache.findOrStore(inst.core, itemsMerge[i][inst.core.idAttribute]);
				inst.core.merge(itemsMerge[i]);
				inst.applyWaitingSuccess();	
						
				//add to collection
				items[limit[0]*limit[1]+i] = inst;
			}
			
		},
		
		/**
		 * Merge an item with current collection (shift all collection elements)
		 * @param {object} item item to merge
		 * @param {object} data data
		 * @param {string} textStatus text status
		 * @param {object} jqXHR XHR object
		 * @memberOf Collection#
		 */
		mergeItem: function(item, data, textStatus, jqXHR){
			//get the position of new element
			var addPosition = data.addPosition;
			//get all key of items in this collection
			var keys = _.keys(this.items).reverse();
			var key = 0;

			//add 1 to each key = or > of addPosition
			for(var i = 0; i < keys.length; i ++){
				key = parseInt(keys[i])
				if(key >= addPosition){
					this.items[key+1] = this.items[key];
				}
			}

			//add to items the new element
			this.items[addPosition] = item;
		},
		
		/**
		 * Unmerge an item at specific position
		 * @param {object} item item to unmerge
		 * @param {object} data data
		 * @param {string} textStatus text status
		 * @param {object} jqXHR XHR object
		 * @memberOf Collection#
		 */
		unmergeItem : function(item, data, textStatus, jqXHR){
			
			//warning no work when item is more than one time in collection
			//remove item
			pos = -1;

			for(var key in this.items){
				
				if(pos > -1){
					
					this.items[pos] = this.items[key];
					delete this.items[key];

					pos = key;

				} else if(this.items[key] == item){
					delete this.items[key];
					pos = key;
				}

			}


		},
		resetItem : function(data, textStatus, jqXHR){
			this.items = {};
			onChangeSizeCollection.call(this, 0);
			onChangeMetaCollection.call(this, data ? data.meta : {});
			
		}
	});
	
	

	var applyViewEvent = function(){
		if(!this.domSource || !this.controller) return;

		var d, ev, ds;

		d = this.controller._domEvs || [];
		ds = this.domSource;

		for(var i = 0; i < d.length; i ++){
			ev = d[i];
			if(ev.sel){
				ds.on(ev.type, ev.sel, domEvInController(this.controller, ev.fn, ev.full)); 
			} else {
				ds.on(ev.type, domEvInController(this.controller, ev.fn, ev.full));
			}
		}
	}

	var unapplyViewEvent = function(){
		if(!this.domSource || !this.controller) return;

		var d, ev, ds;

		d = this.controller._domEvs || [];
		ds = this.domSource;

		for(var i = 0; i < d.length; i ++){
			ev = d[i];

			if(ev.sel){
				ds.off(ev.type, ev.sel);
			} else {
				ds.off(ev.type);
			}
		}
	}

	var viewInjectAsNodeText = function(jNode, message){
		if(jNode){
			if(!_.isArray(message)){
				message = message.split("\n");
			}

			for(var i = 0; i < message.length; i ++){
				jNode.append(document.createTextNode(message[i]));
				if(i+1 < message.length) jNode.append( document.createElement('br'));

			}
		}
	};
	/**
	* Create new instance of view
	* @constructor
	* @global
	*/
	var View = function(selector){
		this.el = null;

		this.setEl(selector || 'body');

		this.source = _.template(this.template());

		this.domSource = null;

		this.events = [];

		this.controller = null;
	};
	/** global public function **/
	/**
	* serach view in dom node. By traversing all parent.
	* @param {object} dom element or jquery object 
	* @return {View|null}
	* @memberOf View
	*/
	View.findUp = function(dom){
		//wrappe dom element
		dom = $(dom);

		while(dom.size() > 0){
			if(dom.get(0).viewHandler){
				return dom.get(0).viewHandler;
			}

			dom = dom.parent();
		}

		return null;
	};

	_.extend(View.prototype, ControllerBinder, {
		/**
		* get source dom attached to this view
		* @memberOf View#
		* @return {object} dom object
		*/
		getSource : function(){
			return this.source;
		},
		/**
		* set source dom on this view
		* @memberOf View#
		* @param {object} source dom source object
		*/
		setSource : function(source){
			this.source = source;
		},
		/**
		* set events list , ! only set not apply
		* @memberOf View#
		* @param {Controller} controller the controller where events and callback are store
		*/
		linkController : function(controller){
			if(this.controller){
				this.unlinkController();
			}

			this.controller = controller;

			if(this.domSource){
				applyViewEvent.call(this);
			}
		},
		/**
		* return controller linked with this view
		* @memberOf View#
		* @return {Controller}
		*/
		getController : function(){
			return this.controller;
		},
		/**
		* unset events list
		* @memberOf View#
		*/
		unlinkController : function(){
			if(this.controller){
				if(this.domSource){
					unapplyViewEvent.call(this);
				}
				this.controller = null;
			}
		},
		/**
		* set root dom element of the view
		* @memberOf View#
		* @param {string|jQuery} selector a jquery selector or a jquery object
		*/
		setEl : function(selector){

			this.el = $(selector);
		},
		/**
		* Template function return the template as string. The mehod need to be replace by extend this peusdo class. The template structur need to have a root nood like xml file
		* @return {string} the template 
		* @memberOf View#
		* @example
			var ItemView = rtsmvc.View.extend({
				template : function(){
					//use array notation for visibility and performance optimisation
					//Array is extend with 2 proxy method.
					//1. push() => p()
					//1. join('') => str()
			
					//template engine is _.template with mustach style delimiter.
					var t = [];
					
					t.p('&lt;li&gt;');
						t.p('&lt;p&gt;{{title}}&lt;/p&gt');
					t.p('&lt;/li&gt');
			
					return t.str();
				}
			});
		*/
		template : function(){
			return '';
		},
		/**
		* Define list of attributes that are security sensitive (provide by user) and inject them as simple text node to avoid xss.
		* @memberOf View#
		* @example
var MyView = rtsmvc.View.extend({
	safeText : {
		//jquery selector : attribute name
		'.content' : 'body'
	},
	template : function(){
		return '&lt;div class=&quot;content&quot;&gt;&lt;/div&gt;';
	}
});
		*/
		safeText : {},
		/**
		* Proxy method of jquery.find for the root el of this view
		* @memberOf View#
		* 
		*/
		find : function(){
			return this.domSource ? this.domSource.find.apply(this.domSource, arguments) : this.el.find.apply(this.el, arguments);
			
		},
		/**
		* Inject in DOM the template string, if instance have inserted element, element will be replace.
		* @memberOf View#
		* @param {object} data template data
		* @param {function} [inject] function that make injection
		* @param {object} defaultData default data for template
		*/
		render : function(data, inject, defaultData){
			data = data || {};
			defaultData = defaultData ||  {};

			for(var key in defaultData){
				if(data[key] === undefined){
					data[key] = defaultData[key];
				}
			} 

			if(this.domSource){
				var newSource = $(this.source(data));

				this.domSource.replaceWith(newSource);

				this.domSource = newSource;

			} else {

				this.domSource = $(this.source(data));
				
				inject = inject || function(){
					//hugly fix for ie 7 and 8, dom is not redraw when injection is done
					this.el.html(this.domSource).hide().show();
					
				};

				inject.call(this)
			}

			//inject safe text data in new nod dom chunck

			for(key in this.safeText){
				viewInjectAsNodeText(this.domSource.find(key), data[this.safeText[key]])
				
			}

			applyViewEvent.call(this);

			//add handler to find this view from dom node
			this.domSource.get(0).viewHandler = this;
		},

		/**
		* Append template to the bottom of the root element
		* @memberOf View#
		* @param {object} data template data
		* @param {object} defaultData default data for template
		*/
		append : function(data, defaultData){
			this.render(data, function(){
				this.el.append(this.domSource).hide().css('display','');
			}, defaultData);
		},

		/**
		* Prepend template to the top of the root element
		* @memberOf View#
		* @param {object} data template data
		* @param {object} defaultData default data for template
		*/
		prepend : function(data, defaultData){
			this.render(data, function(){
				this.el.prepend(this.domSource).hide().css('display','');
			}, defaultData);
		},
		/**
		* Insert view in dom node after the root element
		* @memberOf View#
		* @param {object} data template data
		* @param {object} defaultData default data for template
		*/
		after : function(data, defaultData){
			this.render(data, function(){
				this.el.after(this.domSource).hide().css('display','');
			}, defaultData);	
		},
		/**
		* Insert view in dom node before the root element
		* @memberOf View#
		* @param {object} data template data
		* @param {object} defaultData default data for template
		*/
		before : function(data, defaultData){
			this.render(data, function(){
				this.el.before(this.domSource).hide().css('display','');
			}, defaultData);
		},
		/**
		* Remove view from dom
		* @memberOf View#
		*/
        remove : function(){
			this.domSource.remove();
			this.domSource = null;
		}

	});

	//--- Controller code

	var regexDomEventSpilter = /^([^ ]+) +(.+)$/;
	var regexIsObjectEvent = /^([^ ])+:(.+)$/;

	var setUnsetController = function(prop, replace, namespace){
		if(!replace) return;
		
		unsetController.call(this, prop, namespace);

		replace.onController(this, namespace);


		//appy to object
		if(namespace){
			this[prop+'s'][namespace] = replace;
		} else {
			this[prop] = replace;
		}

		var e = prop + ":set";

		e = namespace ? namespace+":"+e : e;

		this.trigger(e, replace, e);
	};

	var unsetController = function(prop, namespace){

		if((this[prop] && !namespace) || (namespace && this[prop+'s'][namespace])){
			var instMemeber, e;

			if(namespace){
				instMemeber = this[prop+'s'][namespace];
				if(instMemeber){
					delete this[prop+'s'][namespace];
				}

				e = namespace+":"+ prop + ":unset";
			} else {
				instMemeber = this[prop];
				if(instMemeber){
					delete this[prop];
				}

				e = prop + ":unset";
			}

			instMemeber.offController(this, namespace);
			

			this.trigger(e, instMemeber, e);


			
		}
	};

	/**
	* Private function that load callbcaks to a controller
	*/
	var loadEvents = function(){
		var evs, domEvs,cbName, cb, domEv, match, domEv, jSel, full;

		domEvs = this._domEvs || (this._domEvs = []);

		evs = this.events;

		for(var evName in evs){
			cbName = evs[evName];
			cb = this[cbName];

			
			
			if(!evName.match(regexIsObjectEvent)){
				match = evName.match(regexDomEventSpilter);
				if(match){
					full = match[0];
					domEv = match[1];
					jSel = match[2];
				} else {
					domEv = evName;
					full = evName;
					jSel = null;
				}
				domEvs.push({type:domEv, sel : jSel, fn : cb, full:full});
			} else {
				this.on(evName, cb);
			}	
		}

	};
	
	var domEvInController = function(context, callback, fullName){
		
		return function(e){
			// domNode, events, eventsName
			callback.call(context, this, e, fullName);
		}
	};
	
	/**
	 * Controller
	 * @constructor
	 * @global
	 * @desc create a new controller, the construction flow is: 1. create instance var. 2. load events RtsDefined in events object. 3. Instantiate default constructor RtsDefine in constructor object. 4. call method init.
	 */
	var Controller = function(){
		/**
		* model linked with controller, this attribute is readonly, if you want to change this attribute {@link Controller#setModel}
		* @fieldOf Controller#
		* @example
var c = rtsmvc.Controller.extend({
	doSomthing : function(){
		var title = this.model.get('title');
	}
});
		*/
		this.model = null;
		this.models = {};

		this.view = null;
		this.collection = null;
		this.collections = {};

		loadEvents.call(this);

		if(_.isObject(this.construct)){
			for(var key in this.construct){
				switch(key){
					
					case "model":
						this.model = new this.construct[key];
						this.model.onController(this);
					break;


					case "view":
						this.view = new this.construct[key];
						this.view.linkController(this);
					break;


					case "collection":
						this.collection = new this.construct[key];
						this.collection.onController(this);
					break;
				}
			}

			delete this.construct;

		}

		this.init();

		

	};

	var ControllerResetReady = function(type, ns){
		var backEvent = type+":ready";

		backEvent = ns ? ns+":"+backEvent : backEvent;

		var c = this._callbacks || {};

		if(_.isArray(c[backEvent])){

			for(var i = 0; i < c[backEvent].length; i ++){
				if(_.isFunction(c[backEvent][i]))c[backEvent][i].isReadyCalled = false;
			}
		}
	};

	_.extend(Controller.prototype, Events, {
		/**
		* Events link with this controller. Events are linked to function RtsDefined in the pseudo-class definition.
		* format for one event is {"event name" : "function_name", ...}
		* 
		* @memberOf Controller#
		* @example
List of all events available 
{
	"model:reload" : "fname", //( model,  event name) launch when model is reload
	"mode:set" : "fname", //( model,  event name)
	"model:ready" : "fname", //(model, event name) triggered when data of the model is fetch into object.
	"namespace:model:set" : "fname", //(model, event name) namespace prefix is use for extra model set.
	"view:set" : "fname", // (view,  event name)
	"collection:set" : "fname", //(collection, event name)
	"namespace:collection:set" : "fname", //(collection, event name)
	"change:attribute" : "fnname", //(attribute name,  new attribute value, old attribtue value, event name) change model attribute event
	"collection:change:meta:meta_attr" : "fname", (new value, attribute name, old value, event name)
	"collection:load" : "fname", (collection, page, limit, event name)
	"collection:addItemAtFirst" : "fname", (collection, event name)
	"collection:removeItem" : "fname", (collection, event name)
	"click .css_selector" : "fname", //(dom object, event object, event name) dom event, all standard dom event are supported (hover, keyup, keydown, blur, ...)
	"view:unset" : "fname", //(view, event name)
	"model:unset" : "fname", //(model, event name)
	"namespace:model:unset" : "fname", //(model, event name)
	"collection:unset" : "fname", //(collection, event name)
	"namespace:collection:unset" : "fname" //(collection, event name)

}

		*/
		events : {},
		
		/**
		* run when new instance is created
		* @memberOf Controller#
		*/
		init : function(){

		},
		/**
		* get model attached to this controller
		* @memberOf Controller#
		* @param {String} [namespace] 
		* @return {Model}
		*/
		getModel : function(namespace){ 
			return namespace ? this.models[namespace] : this.model;
		},
		/**
		* get view attached to this controller
		* @memberOf Controller#
		* @return {View}
		*/
		getView : function(){ return this.view;},
		/**
		* get collection attached to this controller
		* @memberOf Controller#
		* @param {String} [namespace]
		* @return {Collection}
		*/
		getCollection : function(namespace){ 
			return namespace ? this.collections[namespace] : this.collection;
		},
		/**
		* Attach model to the controller, trigger event model:set
		* @memberOf Controller#
		* @param {Model} model
		* @param {string} [namespace] if provide store the model in a namespace, all event will be prefixed with this namespace 
		*/
		setModel : function(model, namespace){
			
			ControllerResetReady.call(this, "model" ,namespace);

			setUnsetController.call(this, "model", model, namespace);
			model.readyTrigger(this);
		},
		/**
		* Attach view to the controller, trigger event view:set
		* @memberOf Controller#
		* @param {View} view 
		*/
		setView : function(view){
			if(this.view){
				this.view.unlinkController(this);
			}

			setUnsetController.call(this, "view", view);
			
			this.view.linkController(this);
		},
		/**
		* Attach collection to the controller, trigger event controller:set
		* @memberOf Controller#
		* @param {Collection} collection 
		* @param {String} [namespace] collection namespace
		*/
		setCollection : function(collection, namespace){
			setUnsetController.call(this, "collection", collection, namespace);
		},
		/**
		* Detach model from controller, trigger event model:unset 
		* @param {string} [namespace] if provide rdetach model from the namespace if not provide detach main model
		* @memberOf Controller#
		*/
		unsetModel : function(namespace){
			unsetController.call(this, "model", namespace);
		},
		/**
		* Detach view from controller, trigger event view:unset
		* @memberOf Controller#
		*/
		unsetView : function(){
			if(this.view){
				//remove all events
				this.view.unlinkController(this);
			}

			unsetController.call(this, "view");

		},
		/**
		* Detach Collection from controller, trigger event collection:unset or namespace:collection:unset
		* @memberOf Controller#
		* @param {String} [namespace] remove collection from controller
		*/
		unsetCollection : function(namespace){

			unsetController.call(this, "collection", namespace);
		},

		/**
		* @private
		* @memberOf Controller#
		*/
		dispatchModel : function(namespace, action, key, value, oldValue){
			var e = action + ":" + key;

			e = namespace == '_' ? e : namespace+":"+e;
		
			if(key == '*'){
				this.trigger(e, this.model, e);
			} else if(
				key == 'ready' &&
				this._callbacks &&
				_.isArray(this._callbacks[e])
			){
				
				//manual trigger we need to add a flag to know if triggers are already called 
				for(var i = 0; i < this._callbacks[e].length; i++){
					if(_.isFunction(this._callbacks[e][i])){
						
						if(this._callbacks[e][i].isReadyCalled != true){
						   this._callbacks[e][i].isReadyCalled = true;
						   this._callbacks[e][i].call(this, e, key, value, oldValue, e);
						}

					}
				}

				
			} else {
				this.trigger(e, key, value, oldValue, e);
			}
			
		},
		/**
		* @private
		* @memberOf Controller#
		*/
		dispatchCollection : function(ns){
			var args = [];

			for(var i = 0; i < arguments.length; i ++){	
				if(i == 0 ){
					args[i] = arguments[i] == '_' ? "collection:"+arguments[i+1] : arguments[i]+":collection:"+arguments[i+1] ;
					i ++;
					continue;
				}
				args[i-1] = arguments[i];
			}
			
			args.push(args[0]);

			this.trigger.apply(this, args);
		},
		remove : function(){
			if(this.model){
				this.model = null;
			}

			if(this.collection){
				this.collection = null;
			}

			if(this.view){
				this.view.remove();
				this.view = null;
			}

			
		}
	});
	
	/**
	 * Construction of CallbackGroup
	 * @constructor
	 * @global
	 * @example
		var c = new rtsmvc.CallbackGroup();
		
		c.onSuccessAll(function(){
			alert("all elements are loaded");
		});
		
		for(var i = 0; i < 10; i ++){
			var m = new Model();
		
			var jQueryAjaxOption = {};
		
			jQueryAjaxOption.success = function(){
				console.log("model loaded");
			}
		
			c.addItem(jQueryAjaxOption);
		
			m.loadById(i, jQueryAjaxOption);
		}
		//start events handeling
		c.ready();

	 */
	var CallbackGroup = function(){
		this.count = 0;
		this.hasError = false;
		this.isReady = false;

		this.cbSuccess = [];
		this.cbError = [];

		this.cbOnSuccssItem = $.noop;
		this.cbOnSuccssAll = $.noop;


		this.cbOnErrorItem = $.noop;
		this.cbOnError = $.noop;
	}

	var cbgOnSuccessWrapper = function(instance){
		var context = this;

		return function(data, textStatus, jqXHR){
			cbgOnSuccess.call(context, instance, data, textStatus, jqXHR);
		}
	}
	var cbgOnSuccess = function(instance, data, textStatus, jqXHR){
		
		if(this.isReady){
			cbgApplyOnSuccess.call(this, instance, data, textStatus, jqXHR);
		} else {
			this.cbSuccess.push[instance, data,textStatus, jqXHR];
		}
	}
	
	var cbgApplyOnSuccess = function(instance, data, textStatus, jqXHR){
		this.count --;

		this.cbOnSuccssItem(instance, data, textStatus, jqXHR);

		cbgGloablCallback.call(this);

		
	}

	var cbgOnErrorWrapper = function(instance){
		var context = this;

		return function(data, textStatus, jqXHR){
			cbgOnError.call(context, instance, data, textStatus, jqXHR);
		}
	}
	var cbgOnError = function(instance, data, textStatus, jqXHR){
		
		if(this.isReady){
			cbgApplyOnError.call(this, instance, data, textStatus, jqXHR);
		} else {
			this.cbError.push[instance, data,textStatus, jqXHR];
		}
	}
	
	var cbgApplyOnError = function(instance, data, textStatus, jqXHR){
		this.count --;
		this.hasError = true;

		this.cbOnErrorItem(instance, data, textStatus, jqXHR);
		
		cbgGloablCallback.call(this);
	}

	var cbgGloablCallback = function(){
		
		if(this.count <= 0){
			if(this.hasError){
				this.cbOnError();
			} else {
				this.cbOnSuccssAll();
			}
		}
	}

	CallbackGroup.prototype = {

		/**
		* add jquery option to callback group
		* @memberOf CallbackGroup#
		* @param {object} options default jquery ajax option
		* @param {object} instance, object link with this callback
		*/
		addItem : function(options, instance){
			this.count ++;
			
			//mergeFnIfDef.call(this, options, "success", this.mergeAttributes);
			mergeFnIfDef.call(this, options, "success", cbgOnSuccessWrapper.call(this, instance));
			mergeFnIfDef.call(this, options, "error", cbgOnErrorWrapper.call(this, instance));

			

		},
		/**
		* set callback for each item that succeed
		* @memberOf CallbackGroup#
		* @param {function} callback the callback
		*/
		onSuccessItem : function(callback){
			this.cbOnSuccssItem = callback;
		},
		/**
		* set callback when all items are succeeds
		* @memberOf CallbackGroup#
		* @param {function} callback the callback
		*/
		onSuccessAll : function(callback){
			this.cbOnSuccssAll = callback;
		},
		/**
		* set callback for each item that failed
		* @memberOf CallbackGroup#
		* @param {function} callback the callback
		*/
		onErrorItem : function(callback){
			this.cbOnErrorItem = callback;
		},
		/**
		* set callback when one or more items are failed
		* @memberOf CallbackGroup#
		* @param {function} callback the callback
		*/
		onError : function(callback){

			this.cbOnError = callback;
		},

		/**
		* start event raising
		* @memberOf CallbackGroup#
		*/
		ready : function(){
			this.isReady = true;

			if(this.cbSuccess.length > 0){
				for(var i = 0; i < this.cbSuccess.length; i ++){
					cbgApplyOnSuccess.apply(this, this.cbSuccess[i][0], this.cbSuccess[i][1], this.cbSuccess[i][2], this.cbSuccess[i][3]);
				}
			}

			if(this.cbError.length > 0){
				for(var i = 0; i < this.cbError.length; i ++){
					cbgApplyOnError.apply(this, this.cbSuccess[i][0], this.cbSuccess[i][1], this.cbSuccess[i][2], this.cbSuccess[i][3]);
				}
			}
		}
	}
	//expose to the world
	ns.CallbackGroup = CallbackGroup;
	/**
	* Create new pseudo class of Model
	* @memberOf Model
	* @function
	* @example
		var Media = rtsmvc.Model.extend({
			//:idMedia will be replaced by id 
			url:"/controller/uri/:idMedia", //as string value or
			url:function(){ // as function value
				var url = "/controller/uri/:idMedia/type/";
		
				return url + this.get("type")
			},
			//name of id attribute
			idAttribute:'idMedia',
			//RtsDefined attributes
			defAttr:{
				//title is the name of the attribute
				title:{
					//validation rules
					validation:['RtsRequired']
				},
		
				intro : {
					//use for custom callback on live sync attributes
					sync : function() {
						$.ajax(...);	
					}
				},
				message : {
					'default' : 'my default message'
				},
				author : {
					'default' : function(){
						return 'my default message as function return';
					},
					virtual : true // on set param, field name is add at the end of url 
				},
				idMedia : {},
				type : {'default' : 'test'},
				//sample how to create link with other models
				childMediaId : {
					onSet : function(value){
						var m = new Media();
						m.loadById(value);
						this.setLocal('childMedia',  m);
					}
				},
				childMedia : {}
			}
		});
		
		var media1 = new Media();
		media1.loadById(12);
	* 
	*/
	ns.Model = Model;

	/**
	* Create new pseudo class of Collection
	* @memberOf Collection
	* @function
	* @example

		var MediaCollection = rtsmvc.Collection.extend({
			//Model object, all items in this collection are Media {@link Model.extend}
			model:Media,
			//source url
			url:'/public/jsmvc/collection/mediaCollection/'
		});

var medias = new MediaCollection();

	*/
	Collection.extend = extend;
	ns.Collection = Collection;
	/**
	* Create new pseudo class of View
	* @memberOf View
	* @function
	* @example

		var TilteView = rtsmvc.View.extend({
			template : function(){
				return '&lt;h1&gt;{{title}}&lt;/h1&gt;';
			}
		});
		
		var title = new TitleView();
		title.setEl('body');
		title.render({title:"hello world"});

	*/

	View.extend = extend;
	ns.View = View;

	/**
	* Create new pseudo class of Controller
	* @memberOf Controller
	* @function
	* @example
		var MyController = rtsmvc.Controller.extend({
			events : {
				"change:title" : "updateTitle",
				"child:change:title", "updateChildTitle", //namespaced model event
				"view:set", "renderView",
				"click h1", "onClickTitle"
			},
			//use construct to RtsDefine default model, collection, and view constructor
			construct : {
				model : ModelTest, {@link Model}
				collection : CollectionTest, {@link Collection}
				view : View {@link View}
			},
			updateTitle : function(key, value, old){
				this.view.render({title:value});
			},
			renderView : function(){
				this.view.render({title: this.model.get("title") || "default Title"});
			},
			onClickTitle : function(){
				var newTitle = prompt("Please enter the new title", this.model.get("title"));
				if(newTitle){
					this.model.set("title", newTitle);
				}
			}
		});
		
		var myMedia = new Media(); {@link Model.extend}
		var myChildMedia = new Media();
		var myView = new TitleView(); {@link View.extend}
		
		myView.setEl('body');
		
		var myController = new MyController();
		myController.setModel(myMedia);
		myController.setView(myView);

		myController.setMode(myChildMedia, "child");
	*/
	Controller.extend = extend;
	ns.Controller = Controller;


	//expose utils

	ns.Util = Util;

})(rtsmvc);

return rtsmvc;
});
